@@ -2,15 +2,16 @@ from enum import Enum
 from functools import partial
 from itertools import chain, cycle
 from logging import getLogger
-from os import environ, path
+from os import path
 from pprint import pformat
 from re import search
 from string import whitespace
-from typing import Dict, Union, Optional, Iterator, Sequence, List
+from tempfile import gettempdir
+from typing import Dict, Union, Iterator, Sequence, List
 
 from PIL import Image, ImageDraw
 
-from dcspy import LcdInfo
+from dcspy import LcdInfo, LcdButton
 from dcspy.sdk import lcd_sdk
 
 try:
@@ -34,16 +35,14 @@ class Aircraft:
         self.cycle_buttons: Dict[str, Iterator[int]] = {}
         self._debug_img = cycle(chain([f'{x:02}' for x in range(10)], range(10, 99)))
 
-    def button_request(self, button: int, request: str = '\n') -> str:
+    def button_request(self, button: LcdButton, request: str = '\n') -> str:
         """
         Prepare aircraft specific DCS-BIOS request for button pressed.
 
         For G13/G15/G510: 1-4
         For G19 9-15: LEFT = 9, RIGHT = 10, OK = 11, CANCEL = 12, UP = 13, DOWN = 14, MENU = 15
 
-        If button is out of scope new line is return.
-
-        :param button: possible values 1-4
+        :param button: LcdButton Enum
         :param request: valid DCS-BIOS command as string
         :return: ready to send DCS-BIOS request
         """
@@ -56,22 +55,20 @@ class Aircraft:
         """Update display."""
         lcd_sdk.update_display(image)
 
-    def prepare_image(self) -> Optional[Image.Image]:
+    def prepare_image(self) -> Image.Image:
         """
         Prepare image to be sent to correct type of LCD.
 
         :return: image instance ready display on LCD
         """
-        img_for_lcd = {1: partial(Image.new, mode='1', size=(self.lcd.width, self.lcd.height), color=self.lcd.background),
-                       2: partial(Image.new, mode='RGBA', size=(self.lcd.width, self.lcd.height), color=self.lcd.background)}
-        try:
-            img = img_for_lcd[self.lcd.type]()
-            getattr(self, f'draw_for_lcd_type_{self.lcd.type}')(img)
-            img.save(path.join(environ.get('TEMP', ''), f'{self.__class__.__name__}_{next(self._debug_img)}.png'), 'PNG')
-            return img
-        except KeyError as err:
-            LOG.debug(f'Wrong LCD type: {self.lcd} or key: {err}')
-            return None
+        img_for_lcd = {'mono': partial(Image.new, mode='1', size=(self.lcd.width, self.lcd.height), color=self.lcd.background),
+                       'color': partial(Image.new, mode='RGBA', size=(self.lcd.width, self.lcd.height), color=self.lcd.background)}
+
+        lcd_type = self.lcd.type.name.lower()
+        img = img_for_lcd[lcd_type]()
+        getattr(self, f'draw_for_lcd_{lcd_type}')(img)
+        img.save(path.join(gettempdir(), f'{self.__class__.__name__}_{next(self._debug_img)}.png'), 'PNG')
+        return img
 
     def set_bios(self, selector: str, value: str) -> None:
         """
@@ -96,11 +93,11 @@ class Aircraft:
         except KeyError:
             return ''
 
-    def draw_for_lcd_type_1(self, img: Image.Image) -> None:
+    def draw_for_lcd_mono(self, img: Image.Image) -> None:
         """Prepare image for Aircraft for Mono LCD."""
         raise NotImplementedError
 
-    def draw_for_lcd_type_2(self, img: Image.Image) -> None:
+    def draw_for_lcd_color(self, img: Image.Image) -> None:
         """Prepare image for Aircraft for Color LCD."""
         raise NotImplementedError
 
@@ -110,9 +107,9 @@ class Aircraft:
 
         :param btn_name: BIOS button name
         """
-        curr_val = int(self.get_bios(btn_name))
-        max_val = self.bios_data[btn_name]['max_value']
         if not self.cycle_buttons[btn_name]:
+            curr_val = int(self.get_bios(btn_name))
+            max_val = self.bios_data[btn_name]['max_value']
             full_seed = list(range(max_val + 1)) + list(range(max_val - 1, 0, -1)) + list(range(max_val + 1))
             seed = full_seed[curr_val + 1:2 * max_val + curr_val + 1]
             LOG.debug(f'{self.__class__.__name__} {btn_name} full_seed: {full_seed} seed: {seed} curr_val: {curr_val}')
@@ -148,7 +145,11 @@ class FA18Chornet(Aircraft):
             'UFC_OPTION_CUEING_4': {'class': 'StringBuffer', 'args': {'address': 0x742e, 'max_length': 1}, 'value': str()},
             'UFC_OPTION_CUEING_5': {'class': 'StringBuffer', 'args': {'address': 0x7430, 'max_length': 1}, 'value': str()},
             'IFEI_FUEL_DOWN': {'class': 'StringBuffer', 'args': {'address': 0x748a, 'max_length': 6}, 'value': str()},
-            'IFEI_FUEL_UP': {'class': 'StringBuffer', 'args': {'address': 0x7490, 'max_length': 6}, 'value': str()}}
+            'IFEI_FUEL_UP': {'class': 'StringBuffer', 'args': {'address': 0x7490, 'max_length': 6}, 'value': str()},
+            'HUD_ATT_SW': {'class': 'IntegerBuffer', 'args': {'address': 0x742e, 'mask': 0x300, 'shift_by': 0x8}, 'value': int(), 'max_value': 2},
+            'IFEI_DWN_BTN': {'class': 'IntegerBuffer', 'args': {'address': 0x7466, 'mask': 0x10, 'shift_by': 0x4}, 'value': int(), 'max_value': 1},
+            'IFEI_UP_BTN': {'class': 'IntegerBuffer', 'args': {'address': 0x7466, 'mask': 0x8, 'shift_by': 0x3}, 'value': int(), 'max_value': 1}}
+        self.cycle_buttons = {'HUD_ATT_SW': '', 'IFEI_DWN_BTN': '', 'IFEI_UP_BTN': ''}  # type: ignore
 
     def _draw_common_data(self, draw: ImageDraw, scale: int) -> ImageDraw:
         scratch_1 = self.get_bios("UFC_SCRATCHPAD_STRING_1_DISPLAY")
@@ -173,11 +174,11 @@ class FA18Chornet(Aircraft):
         draw.text(xy=(36 * scale, 29 * scale), text=self.get_bios('IFEI_FUEL_UP'), fill=self.lcd.foreground, font=self.lcd.font_l)
         return draw
 
-    def draw_for_lcd_type_1(self, img: Image.Image) -> None:
+    def draw_for_lcd_mono(self, img: Image.Image) -> None:
         """Prepare image for F/A-18C Hornet for Mono LCD."""
         self._draw_common_data(draw=ImageDraw.Draw(img), scale=1)
 
-    def draw_for_lcd_type_2(self, img: Image.Image) -> None:
+    def draw_for_lcd_color(self, img: Image.Image) -> None:
         """Prepare image for F/A-18C Hornet for Color LCD."""
         draw = self._draw_common_data(draw=ImageDraw.Draw(img), scale=2)
         draw.text(xy=(72, 100), text=self.get_bios('IFEI_FUEL_DOWN'), fill=self.lcd.foreground, font=self.lcd.font_l)
@@ -194,27 +195,34 @@ class FA18Chornet(Aircraft):
             value = value.replace('`', '1').replace('~', '2')
         super().set_bios(selector, value)
 
-    def button_request(self, button: int, request: str = '\n') -> str:
+    def button_request(self, button: LcdButton, request: str = '\n') -> str:
         """
         Prepare F/A-18 Hornet specific DCS-BIOS request for button pressed.
 
         For G13/G15/G510: 1-4
         For G19 9-15: LEFT = 9, RIGHT = 10, OK = 11, CANCEL = 12, UP = 13, DOWN = 14, MENU = 15
-        If button is out of scope new line is return.
-        :param button: possible values 1-4
+
+        :param button: LcdButton Enum
         :param request: valid DCS-BIOS command as string
         :return: ready to send DCS-BIOS request
         """
-        action = {1: 'UFC_COMM1_CHANNEL_SELECT DEC\n',
-                  2: 'UFC_COMM1_CHANNEL_SELECT INC\n',
-                  3: 'UFC_COMM2_CHANNEL_SELECT DEC\n',
-                  4: 'UFC_COMM2_CHANNEL_SELECT INC\n',
-                  9: 'UFC_COMM1_CHANNEL_SELECT DEC\n',
-                  10: 'UFC_COMM1_CHANNEL_SELECT INC\n',
-                  14: 'UFC_COMM2_CHANNEL_SELECT DEC\n',
-                  13: 'UFC_COMM2_CHANNEL_SELECT INC\n',
-                  15: 'IFEI_DWN_BTN 1\nIFEI_DWN_BTN 0\n',
-                  12: 'IFEI_UP_BTN 1\nIFEI_UP_BTN 0\n'}
+        button_map = {LcdButton.OK: 'HUD_ATT_SW', LcdButton.CANCEL: 'IFEI_UP_BTN', LcdButton.MENU: 'IFEI_DWN_BTN'}
+        settings = 0
+        button_bios_name = ''
+        if button in button_map:
+            button_bios_name = button_map[button]
+            settings = self.get_next_value_for_button(button_bios_name)
+        action = {LcdButton.ONE: 'UFC_COMM1_CHANNEL_SELECT DEC\n',
+                  LcdButton.TWO: 'UFC_COMM1_CHANNEL_SELECT INC\n',
+                  LcdButton.THREE: 'UFC_COMM2_CHANNEL_SELECT DEC\n',
+                  LcdButton.FOUR: 'UFC_COMM2_CHANNEL_SELECT INC\n',
+                  LcdButton.LEFT: 'UFC_COMM1_CHANNEL_SELECT DEC\n',
+                  LcdButton.RIGHT: 'UFC_COMM1_CHANNEL_SELECT INC\n',
+                  LcdButton.DOWN: 'UFC_COMM2_CHANNEL_SELECT DEC\n',
+                  LcdButton.UP: 'UFC_COMM2_CHANNEL_SELECT INC\n',
+                  LcdButton.MENU: f'{button_bios_name} {settings}\n',
+                  LcdButton.CANCEL: f'{button_bios_name} {settings}\n',
+                  LcdButton.OK: f'{button_bios_name} {settings}\n'}
         return super().button_request(button, action.get(button, '\n'))
 
 
@@ -227,11 +235,11 @@ class F16C50(Aircraft):
         """
         super().__init__(lcd_type)
         self.bios_data: Dict[str, BIOS_VALUE] = {
-            'DED_LINE_1': {'class': 'StringBuffer', 'args': {'address': 0x4504, 'max_length': 29}, 'value': str()},
-            'DED_LINE_2': {'class': 'StringBuffer', 'args': {'address': 0x4522, 'max_length': 29}, 'value': str()},
-            'DED_LINE_3': {'class': 'StringBuffer', 'args': {'address': 0x4540, 'max_length': 29}, 'value': str()},
-            'DED_LINE_4': {'class': 'StringBuffer', 'args': {'address': 0x455e, 'max_length': 29}, 'value': str()},
-            'DED_LINE_5': {'class': 'StringBuffer', 'args': {'address': 0x457c, 'max_length': 29}, 'value': str()},
+            'DED_LINE_1': {'class': 'StringBuffer', 'args': {'address': 0x450a, 'max_length': 29}, 'value': str()},
+            'DED_LINE_2': {'class': 'StringBuffer', 'args': {'address': 0x4528, 'max_length': 29}, 'value': str()},
+            'DED_LINE_3': {'class': 'StringBuffer', 'args': {'address': 0x4546, 'max_length': 29}, 'value': str()},
+            'DED_LINE_4': {'class': 'StringBuffer', 'args': {'address': 0x4564, 'max_length': 29}, 'value': str()},
+            'DED_LINE_5': {'class': 'StringBuffer', 'args': {'address': 0x4582, 'max_length': 29}, 'value': str()},
             'IFF_MASTER_KNB': {'class': 'IntegerBuffer', 'args': {'address': 0x4450, 'mask': 0xe, 'shift_by': 0x1}, 'value': int(), 'max_value': 4},
             'IFF_ENABLE_SW': {'class': 'IntegerBuffer', 'args': {'address': 0x4450, 'mask': 0x600, 'shift_by': 0x9}, 'value': int(), 'max_value': 2},
             'IFF_M4_CODE_SW': {'class': 'IntegerBuffer', 'args': {'address': 0x4450, 'mask': 0x30, 'shift_by': 0x4}, 'value': int(), 'max_value': 2},
@@ -243,11 +251,11 @@ class F16C50(Aircraft):
             offset = (i - 1) * 8 * scale
             draw.text(xy=(0, offset), text=self.get_bios(f'DED_LINE_{i}'), fill=self.lcd.foreground, font=self.lcd.font_s)
 
-    def draw_for_lcd_type_1(self, img: Image.Image) -> None:
+    def draw_for_lcd_mono(self, img: Image.Image) -> None:
         """Prepare image for F-16C Viper for Mono LCD."""
         self._draw_common_data(draw=ImageDraw.Draw(img), scale=1)
 
-    def draw_for_lcd_type_2(self, img: Image.Image) -> None:
+    def draw_for_lcd_color(self, img: Image.Image) -> None:
         """Prepare image for F-16C Viper for Color LCD."""
         self._draw_common_data(draw=ImageDraw.Draw(img), scale=2)
 
@@ -269,23 +277,42 @@ class F16C50(Aircraft):
             value = value.replace('\u0002', '')  # List - 7
             value = value.replace('\x80', '')  # 1 T-ILS
             value = value.replace('\x08', '')  # 7 MARK
+            value = value.replace('*', '\u25d9')  # INVERSE WHITE CIRCLE
         super().set_bios(selector, value)
 
-    def button_request(self, button: int, request: str = '\n') -> str:
+    def button_request(self, button: LcdButton, request: str = '\n') -> str:
         """
         Prepare F-16C Viper specific DCS-BIOS request for button pressed.
 
         For G13/G15/G510: 1-4
         For G19 9-15: LEFT = 9, RIGHT = 10, OK = 11, CANCEL = 12, UP = 13, DOWN = 14, MENU = 15
-        :param button: possible values 1-4
+
+        :param button: LcdButton Enum
         :param request: valid DCS-BIOS command as string
         :return: ready to send DCS-BIOS request
         """
-        button_map = {1: 'IFF_MASTER_KNB', 2: 'IFF_ENABLE_SW', 3: 'IFF_M4_CODE_SW', 4: 'IFF_M4_REPLY_SW',
-                      9: 'IFF_MASTER_KNB', 10: 'IFF_ENABLE_SW', 14: 'IFF_M4_CODE_SW', 13: 'IFF_M4_REPLY_SW'}
-        button_bios_name = button_map[button]
-        setting = self.get_next_value_for_button(button_bios_name)
-        return super().button_request(button, f'{button_bios_name} {setting}\n')
+        button_map = {LcdButton.ONE: 'IFF_MASTER_KNB',
+                      LcdButton.TWO: 'IFF_ENABLE_SW',
+                      LcdButton.THREE: 'IFF_M4_CODE_SW',
+                      LcdButton.FOUR: 'IFF_M4_REPLY_SW',
+                      LcdButton.LEFT: 'IFF_MASTER_KNB',
+                      LcdButton.RIGHT: 'IFF_ENABLE_SW',
+                      LcdButton.DOWN: 'IFF_M4_CODE_SW',
+                      LcdButton.UP: 'IFF_M4_REPLY_SW'}
+        settings = 0
+        button_bios_name = ''
+        if button in button_map:
+            button_bios_name = button_map[button]
+            settings = self.get_next_value_for_button(button_bios_name)
+        action = {LcdButton.ONE: f'{button_bios_name} {settings}\n',
+                  LcdButton.TWO: f'{button_bios_name} {settings}\n',
+                  LcdButton.THREE: f'{button_bios_name} {settings}\n',
+                  LcdButton.FOUR: f'{button_bios_name} {settings}\n',
+                  LcdButton.LEFT: f'{button_bios_name} {settings}\n',
+                  LcdButton.RIGHT: f'{button_bios_name} {settings}\n',
+                  LcdButton.DOWN: f'{button_bios_name} {settings}\n',
+                  LcdButton.UP: f'{button_bios_name} {settings}\n'}
+        return super().button_request(button, action.get(button, '\n'))
 
 
 class Ka50(Aircraft):
@@ -313,28 +340,7 @@ class Ka50(Aircraft):
             'AP_HDG_HOLD_LED': {'class': 'IntegerBuffer', 'args': {'address': 0x1936, 'mask': 0x800, 'shift_by': 0xb}, 'value': int()},
             'AP_PITCH_HOLD_LED': {'class': 'IntegerBuffer', 'args': {'address': 0x1936, 'mask': 0x2000, 'shift_by': 0xd}, 'value': int()}}
 
-    def button_request(self, button: int, request: str = '\n') -> str:
-        """
-        Prepare Ka-50 Black Shark specific DCS-BIOS request for button pressed.
-
-        For G13/G15/G510: 1-4
-        For G19 9-15: LEFT = 9, RIGHT = 10, OK = 11, CANCEL = 12, UP = 13, DOWN = 14, MENU = 15
-        If button is out of scope new line is return.
-        :param button: possible values 1-4
-        :param request: valid DCS-BIOS command as string
-        :return: ready to send DCS-BIOS request
-        """
-        action = {1: 'PVI_WAYPOINTS_BTN 1\nPVI_WAYPOINTS_BTN 0\n',
-                  2: 'PVI_FIXPOINTS_BTN 1\nPVI_FIXPOINTS_BTN 0\n',
-                  3: 'PVI_AIRFIELDS_BTN 1\nPVI_AIRFIELDS_BTN 0\n',
-                  4: 'PVI_TARGETS_BTN 1\nPVI_TARGETS_BTN 0\n',
-                  9: 'PVI_WAYPOINTS_BTN 1\nPVI_WAYPOINTS_BTN 0\n',
-                  10: 'PVI_FIXPOINTS_BTN 1\nPVI_FIXPOINTS_BTN 0\n',
-                  14: 'PVI_AIRFIELDS_BTN 1\nPVI_AIRFIELDS_BTN 0\n',
-                  13: 'PVI_TARGETS_BTN 1\nPVI_TARGETS_BTN 0\n'}
-        return super().button_request(button, action.get(button, '\n'))
-
-    def _auto_pilot_switch_1(self, draw_obj: ImageDraw) -> None:
+    def _auto_pilot_switch_mono(self, draw_obj: ImageDraw) -> None:
         """
         Draw rectangle and add text for autopilot channels in correct coordinates.
 
@@ -347,7 +353,7 @@ class Ka50(Aircraft):
                                                     ((128, 22, 141, 39), (130, 24), 'A', self.get_bios('AP_ALT_HOLD_LED'))):
             self._draw_autopilot_channels(ap_channel, c_rect, c_text, draw_obj, turn_on)
 
-    def _auto_pilot_switch_2(self, draw_obj: ImageDraw) -> None:
+    def _auto_pilot_switch_color(self, draw_obj: ImageDraw) -> None:
         """
         Draw rectangle and add text for autopilot channels in correct coordinates.
 
@@ -368,7 +374,7 @@ class Ka50(Aircraft):
             draw_obj.rectangle(xy=c_rect, fill=self.lcd.background, outline=self.lcd.foreground)
             draw_obj.text(xy=c_text, text=ap_channel, fill=self.lcd.foreground, font=self.lcd.font_l)
 
-    def draw_for_lcd_type_1(self, img: Image.Image) -> None:
+    def draw_for_lcd_mono(self, img: Image.Image) -> None:
         """Prepare image for Ka-50 Black Shark for Mono LCD."""
         draw = ImageDraw.Draw(img)
         for rect_xy in [(0, 1, 85, 18), (0, 22, 85, 39), (88, 1, 103, 18), (88, 22, 103, 39)]:
@@ -376,9 +382,9 @@ class Ka50(Aircraft):
         line1, line2 = self._generate_pvi_lines()
         draw.text(xy=(2, 3), text=line1, fill=self.lcd.foreground, font=self.lcd.font_l)
         draw.text(xy=(2, 24), text=line2, fill=self.lcd.foreground, font=self.lcd.font_l)
-        self._auto_pilot_switch_1(draw)
+        self._auto_pilot_switch_mono(draw)
 
-    def draw_for_lcd_type_2(self, img: Image.Image) -> None:
+    def draw_for_lcd_color(self, img: Image.Image) -> None:
         """Prepare image for Ka-50 Black Shark for Mono LCD."""
         draw = ImageDraw.Draw(img)
         for rect_xy in [(0, 2, 170, 36), (0, 44, 170, 78), (176, 2, 206, 36), (176, 44, 203, 78)]:
@@ -386,7 +392,7 @@ class Ka50(Aircraft):
         line1, line2 = self._generate_pvi_lines()
         draw.text(xy=(4, 6), text=line1, fill=self.lcd.foreground, font=self.lcd.font_l)
         draw.text(xy=(4, 48), text=line2, fill=self.lcd.foreground, font=self.lcd.font_l)
-        self._auto_pilot_switch_2(draw)
+        self._auto_pilot_switch_color(draw)
 
     def _generate_pvi_lines(self) -> Sequence[str]:
         text1, text2 = '', ''
@@ -404,6 +410,27 @@ class Ka50(Aircraft):
         line2 = f'{self.get_bios("PVI_LINE2_SIGN")}{text2} {self.get_bios("PVI_LINE2_POINT")}'
         return line1, line2
 
+    def button_request(self, button: LcdButton, request: str = '\n') -> str:
+        """
+        Prepare Ka-50 Black Shark specific DCS-BIOS request for button pressed.
+
+        For G13/G15/G510: 1-4
+        For G19 9-15: LEFT = 9, RIGHT = 10, OK = 11, CANCEL = 12, UP = 13, DOWN = 14, MENU = 15
+
+        :param button: LcdButton Enum
+        :param request: valid DCS-BIOS command as string
+        :return: ready to send DCS-BIOS request
+        """
+        action = {LcdButton.ONE: 'PVI_WAYPOINTS_BTN 1\nPVI_WAYPOINTS_BTN 0\n',
+                  LcdButton.TWO: 'PVI_FIXPOINTS_BTN 1\nPVI_FIXPOINTS_BTN 0\n',
+                  LcdButton.THREE: 'PVI_AIRFIELDS_BTN 1\nPVI_AIRFIELDS_BTN 0\n',
+                  LcdButton.FOUR: 'PVI_TARGETS_BTN 1\nPVI_TARGETS_BTN 0\n',
+                  LcdButton.LEFT: 'PVI_WAYPOINTS_BTN 1\nPVI_WAYPOINTS_BTN 0\n',
+                  LcdButton.RIGHT: 'PVI_FIXPOINTS_BTN 1\nPVI_FIXPOINTS_BTN 0\n',
+                  LcdButton.DOWN: 'PVI_AIRFIELDS_BTN 1\nPVI_AIRFIELDS_BTN 0\n',
+                  LcdButton.UP: 'PVI_TARGETS_BTN 1\nPVI_TARGETS_BTN 0\n'}
+        return super().button_request(button, action.get(button, '\n'))
+
 
 class ApacheEufdMode(Enum):
     IDM = 1
@@ -411,7 +438,7 @@ class ApacheEufdMode(Enum):
     PRE = 4
 
 
-class AH64DBLKII(Aircraft):
+class AH64D(Aircraft):
     def __init__(self, lcd_type: LcdInfo) -> None:
         """
         Create AH-64D Apache.
@@ -422,27 +449,27 @@ class AH64DBLKII(Aircraft):
         self.mode = ApacheEufdMode.IDM
         self.warning_line = 1
         self.bios_data: Dict[str, BIOS_VALUE] = {
-            'PLT_EUFD_LINE1': {'class': 'StringBuffer', 'args': {'address': 0x80c0, 'max_length': 56}, 'value': str()},
-            'PLT_EUFD_LINE2': {'class': 'StringBuffer', 'args': {'address': 0x80f8, 'max_length': 56}, 'value': str()},
-            'PLT_EUFD_LINE3': {'class': 'StringBuffer', 'args': {'address': 0x8130, 'max_length': 56}, 'value': str()},
-            'PLT_EUFD_LINE4': {'class': 'StringBuffer', 'args': {'address': 0x8168, 'max_length': 56}, 'value': str()},
-            'PLT_EUFD_LINE5': {'class': 'StringBuffer', 'args': {'address': 0x81a0, 'max_length': 56}, 'value': str()},
-            'PLT_EUFD_LINE6': {'class': 'StringBuffer', 'args': {'address': 0x81d8, 'max_length': 56}, 'value': str()},
-            'PLT_EUFD_LINE7': {'class': 'StringBuffer', 'args': {'address': 0x8210, 'max_length': 56}, 'value': str()},
-            'PLT_EUFD_LINE8': {'class': 'StringBuffer', 'args': {'address': 0x8248, 'max_length': 56}, 'value': str()},
-            'PLT_EUFD_LINE9': {'class': 'StringBuffer', 'args': {'address': 0x8280, 'max_length': 56}, 'value': str()},
-            'PLT_EUFD_LINE10': {'class': 'StringBuffer', 'args': {'address': 0x82b8, 'max_length': 56}, 'value': str()},
-            'PLT_EUFD_LINE11': {'class': 'StringBuffer', 'args': {'address': 0x82f0, 'max_length': 56}, 'value': str()},
-            'PLT_EUFD_LINE12': {'class': 'StringBuffer', 'args': {'address': 0x8328, 'max_length': 56}, 'value': str()},
-            'PLT_EUFD_LINE14': {'class': 'StringBuffer', 'args': {'address': 0x8398, 'max_length': 56}, 'value': str()},
+            'PLT_EUFD_LINE1': {'class': 'StringBuffer', 'args': {'address': 0x80c2, 'max_length': 56}, 'value': str()},
+            'PLT_EUFD_LINE2': {'class': 'StringBuffer', 'args': {'address': 0x80fa, 'max_length': 56}, 'value': str()},
+            'PLT_EUFD_LINE3': {'class': 'StringBuffer', 'args': {'address': 0x8132, 'max_length': 56}, 'value': str()},
+            'PLT_EUFD_LINE4': {'class': 'StringBuffer', 'args': {'address': 0x816a, 'max_length': 56}, 'value': str()},
+            'PLT_EUFD_LINE5': {'class': 'StringBuffer', 'args': {'address': 0x81a2, 'max_length': 56}, 'value': str()},
+            'PLT_EUFD_LINE6': {'class': 'StringBuffer', 'args': {'address': 0x81da, 'max_length': 56}, 'value': str()},
+            'PLT_EUFD_LINE7': {'class': 'StringBuffer', 'args': {'address': 0x8212, 'max_length': 56}, 'value': str()},
+            'PLT_EUFD_LINE8': {'class': 'StringBuffer', 'args': {'address': 0x824a, 'max_length': 56}, 'value': str()},
+            'PLT_EUFD_LINE9': {'class': 'StringBuffer', 'args': {'address': 0x8282, 'max_length': 56}, 'value': str()},
+            'PLT_EUFD_LINE10': {'class': 'StringBuffer', 'args': {'address': 0x82ba, 'max_length': 56}, 'value': str()},
+            'PLT_EUFD_LINE11': {'class': 'StringBuffer', 'args': {'address': 0x82f2, 'max_length': 56}, 'value': str()},
+            'PLT_EUFD_LINE12': {'class': 'StringBuffer', 'args': {'address': 0x832a, 'max_length': 56}, 'value': str()},
+            'PLT_EUFD_LINE14': {'class': 'StringBuffer', 'args': {'address': 0x839a, 'max_length': 56}, 'value': str()},
         }
 
-    def draw_for_lcd_type_1(self, img: Image.Image) -> None:
+    def draw_for_lcd_mono(self, img: Image.Image) -> None:
         """Prepare image for AH-64D Apache for Mono LCD."""
         LOG.debug(f'Mode: {self.mode}')
         getattr(self, f'_draw_for_{self.mode.name.lower()}')(draw=ImageDraw.Draw(img), scale=1)
 
-    def draw_for_lcd_type_2(self, img: Image.Image) -> None:
+    def draw_for_lcd_color(self, img: Image.Image) -> None:
         """Prepare image for AH-64D Apache for Color LCD."""
         LOG.debug(f'Mode: {self.mode}')
         getattr(self, f'_draw_for_{self.mode.name.lower()}')(draw=ImageDraw.Draw(img), scale=2)
@@ -487,12 +514,13 @@ class AH64DBLKII(Aircraft):
                       9: r'\s*\|([\u2192\s][A-Z\d\/]*)\s*([\d\.]*)\s+',
                       10: r'\s*\|([\u2192\s][A-Z\d\/]*)\s*([\d\.]*)\s+',
                       11: r'\s*\|([\u2192\s][A-Z\d\/]*)\s*([\d\.]*)\s+'}
-        for i, x, y in zip(range(2, 12),
-                           [0, 0, 0, 0, 0, 80, 80, 80, 80, 80],
-                           [j * 8 * scale for j in range(0, 5)] * 2):
+        for i, xcord, ycord in zip(range(2, 12),
+                                   [0, 0, 0, 0, 0, 80 * scale, 80 * scale, 80 * scale, 80 * scale, 80 * scale],
+                                   [j * 8 * scale for j in range(0, 5)] * 2):
             mat = search(match_dict[i], self.get_bios(f'PLT_EUFD_LINE{i}'))
             if mat:
-                draw.text(xy=(x, y), text=f'{mat.group(1):<9}{mat.group(2):>7}', fill=self.lcd.foreground, font=self.lcd.font_xs)
+                draw.text(xy=(xcord, ycord), text=f'{mat.group(1):<9}{mat.group(2):>7}',
+                          fill=self.lcd.foreground, font=self.lcd.font_xs)
 
     def set_bios(self, selector: str, value: str) -> None:
         """
@@ -515,37 +543,37 @@ class AH64DBLKII(Aircraft):
             value = value.replace('!', '\u2192')  # replace ! with ->
         super().set_bios(selector, value)
 
-    def button_request(self, button: int, request: str = '\n') -> str:
+    def button_request(self, button: LcdButton, request: str = '\n') -> str:
         """
         Prepare AH-64D Apache specific DCS-BIOS request for button pressed.
 
         For G13/G15/G510: 1-4
         For G19 9-15: LEFT = 9, RIGHT = 10, OK = 11, CANCEL = 12, UP = 13, DOWN = 14, MENU = 15
-        If button is out of scope new line is return.
-        :param button: possible values 1-4
+
+        :param button: LcdButton Enum
         :param request: valid DCS-BIOS command as string
         :return: ready to send DCS-BIOS request
         """
-        wca_or_idm = f'PLT_EUFD_WCA 0\nPLT_EUFD_WCA 1\n'
+        wca_or_idm = 'PLT_EUFD_WCA 0\nPLT_EUFD_WCA 1\n'
         if self.mode == ApacheEufdMode.IDM:
-            wca_or_idm = f'PLT_EUFD_IDM 0\nPLT_EUFD_IDM 1\n'
+            wca_or_idm = 'PLT_EUFD_IDM 0\nPLT_EUFD_IDM 1\n'
 
-        if button in (4, 13) and self.mode == ApacheEufdMode.IDM:
+        if button in (LcdButton.FOUR, LcdButton.UP) and self.mode == ApacheEufdMode.IDM:
             self.mode = ApacheEufdMode.WCA
-        elif button in (4, 13) and self.mode != ApacheEufdMode.IDM:
+        elif button in (LcdButton.FOUR, LcdButton.UP) and self.mode != ApacheEufdMode.IDM:
             self.mode = ApacheEufdMode.IDM
 
-        if button in (1, 9) and self.mode == ApacheEufdMode.WCA:
+        if button in (LcdButton.ONE, LcdButton.LEFT) and self.mode == ApacheEufdMode.WCA:
             self.warning_line += 1
 
-        action = {1: wca_or_idm,
-                  2: 'PLT_EUFD_RTS 0\nPLT_EUFD_RTS 1\n',
-                  3: 'PLT_EUFD_PRESET 0\nPLT_EUFD_PRESET 1\n',
-                  4: 'PLT_EUFD_ENT 0\nPLT_EUFD_ENT 1\n',
-                  9: wca_or_idm,
-                  10: 'PLT_EUFD_RTS 0\nPLT_EUFD_RTS 1\n',
-                  14: 'PLT_EUFD_PRESET 0\nPLT_EUFD_PRESET 1\n',
-                  13: 'PLT_EUFD_ENT 0\nPLT_EUFD_ENT 1\n'}
+        action = {LcdButton.ONE: wca_or_idm,
+                  LcdButton.TWO: 'PLT_EUFD_RTS 0\nPLT_EUFD_RTS 1\n',
+                  LcdButton.THREE: 'PLT_EUFD_PRESET 0\nPLT_EUFD_PRESET 1\n',
+                  LcdButton.FOUR: 'PLT_EUFD_ENT 0\nPLT_EUFD_ENT 1\n',
+                  LcdButton.LEFT: wca_or_idm,
+                  LcdButton.RIGHT: 'PLT_EUFD_RTS 0\nPLT_EUFD_RTS 1\n',
+                  LcdButton.DOWN: 'PLT_EUFD_PRESET 0\nPLT_EUFD_PRESET 1\n',
+                  LcdButton.UP: 'PLT_EUFD_ENT 0\nPLT_EUFD_ENT 1\n'}
         return super().button_request(button, action.get(button, '\n'))
 
 
@@ -581,7 +609,7 @@ class A10C(Aircraft):
               f'{self.get_bios("UHF_POINT1MHZ_SEL")}{self.get_bios("UHF_POINT25_SEL")}'
         return uhf, vhfam, vhffm
 
-    def draw_for_lcd_type_1(self, img: Image.Image) -> None:
+    def draw_for_lcd_mono(self, img: Image.Image) -> None:
         """Prepare image for A-10C Warthog or A-10C II Tank Killer for Mono LCD."""
         draw = ImageDraw.Draw(img)
         uhf, vhfam, vhffm = self._generate_freq_values()
@@ -590,7 +618,7 @@ class A10C(Aircraft):
             offset = i * 10
             draw.text(xy=(0, offset), text=line, fill=self.lcd.foreground, font=self.lcd.font_s)
 
-    def draw_for_lcd_type_2(self, img: Image.Image) -> None:
+    def draw_for_lcd_color(self, img: Image.Image) -> None:
         """Prepare image for A-10C Warthog or A-10C II Tank Killer for Color LCD."""
         draw = ImageDraw.Draw(img)
         uhf, vhfam, vhffm = self._generate_freq_values()
@@ -618,38 +646,42 @@ class F14B(Aircraft):
             'RIO_CAP_NE': {'class': 'IntegerBuffer', 'args': {'address': 0x12c4, 'mask': 0x1000, 'shift_by': 0xc}, 'value': int()},
             'RIO_CAP_ENTER': {'class': 'IntegerBuffer', 'args': {'address': 0x12c4, 'mask': 0x8000, 'shift_by': 0xf}, 'value': int()}}
 
-    def button_request(self, button: int, request: str = '\n') -> str:
-        """
-        Prepare F-14 Tomcat specific DCS-BIOS request for button pressed.
-
-        For G13/G15/G510: 1-4
-        For G19 9-15: LEFT = 9, RIGHT = 10, OK = 11, CANCEL = 12, UP = 13, DOWN = 14, MENU = 15
-        If button is out of scope new line is return.
-        :param button: possible values 1-4
-        :param request: valid DCS-BIOS command as string
-        :return: ready to send DCS-BIOS request
-        """
-        action = {1: 'RIO_CAP_CLEAR 1\nRIO_CAP_CLEAR 0\n',
-                  2: 'RIO_CAP_SW 1\nRIO_CAP_SW 0\n',
-                  3: 'RIO_CAP_NE 1\nRIO_CAP_NE 0\n',
-                  4: 'RIO_CAP_ENTER 1\nRIO_CAP_ENTER 0\n',
-                  9: 'RIO_CAP_CLEAR 1\nRIO_CAP_CLEAR 0\n',
-                  10: 'RIO_CAP_SW 1\nRIO_CAP_SW 0\n',
-                  14: 'RIO_CAP_NE 1\nRIO_CAP_NE 0\n',
-                  13: 'RIO_CAP_ENTER 1\nRIO_CAP_ENTER 0\n'}
-        return super().button_request(button, action.get(button, '\n'))
-
     def _draw_common_data(self, draw: ImageDraw) -> None:
         draw.text(xy=(2, 3), text='F-14B Tomcat', fill=self.lcd.foreground, font=self.lcd.font_l)
 
-    def draw_for_lcd_type_1(self, img: Image.Image) -> None:
+    def draw_for_lcd_mono(self, img: Image.Image) -> None:
         """Prepare image for F-14B Tomcat for Mono LCD."""
         self._draw_common_data(draw=ImageDraw.Draw(img))
 
-    def draw_for_lcd_type_2(self, img: Image.Image) -> None:
+    def draw_for_lcd_color(self, img: Image.Image) -> None:
         """Prepare image for F-14B Tomcat for Color LCD."""
         self._draw_common_data(draw=ImageDraw.Draw(img))
 
+    def button_request(self, button: LcdButton, request: str = '\n') -> str:
+        """
+        Prepare F-14B Tomcat specific DCS-BIOS request for button pressed.
+
+        For G13/G15/G510: 1-4
+        For G19 9-15: LEFT = 9, RIGHT = 10, OK = 11, CANCEL = 12, UP = 13, DOWN = 14, MENU = 15
+
+        :param button: LcdButton Enum
+        :param request: valid DCS-BIOS command as string
+        :return: ready to send DCS-BIOS request
+        """
+        action = {LcdButton.ONE: 'RIO_CAP_CLEAR 1\nRIO_CAP_CLEAR 0\n',
+                  LcdButton.TWO: 'RIO_CAP_SW 1\nRIO_CAP_SW 0\n',
+                  LcdButton.THREE: 'RIO_CAP_NE 1\nRIO_CAP_NE 0\n',
+                  LcdButton.FOUR: 'RIO_CAP_ENTER 1\nRIO_CAP_ENTER 0\n',
+                  LcdButton.LEFT: 'RIO_CAP_CLEAR 1\nRIO_CAP_CLEAR 0\n',
+                  LcdButton.RIGHT: 'RIO_CAP_SW 1\nRIO_CAP_SW 0\n',
+                  LcdButton.DOWN: 'RIO_CAP_NE 1\nRIO_CAP_NE 0\n',
+                  LcdButton.UP: 'RIO_CAP_ENTER 1\nRIO_CAP_ENTER 0\n'}
+        return super().button_request(button, action.get(button, '\n'))
+
+
+class F14A135GR(F14B):
+    pass
+
 
 class AV8BNA(Aircraft):
     def __init__(self, lcd_type: LcdInfo) -> None:
@@ -690,30 +722,31 @@ class AV8BNA(Aircraft):
                       text=f'{i}{self.get_bios(f"AV8BNA_ODU_{i}_SELECT")}{self.get_bios(f"AV8BNA_ODU_{i}_Text")}')
         return draw
 
-    def draw_for_lcd_type_1(self, img: Image.Image) -> None:
+    def draw_for_lcd_mono(self, img: Image.Image) -> None:
         """Prepare image for AV-8B N/A for Mono LCD."""
         self._draw_common_data(draw=ImageDraw.Draw(img), scale=1)
 
-    def draw_for_lcd_type_2(self, img: Image.Image) -> None:
+    def draw_for_lcd_color(self, img: Image.Image) -> None:
         """Prepare image for AV-8B N/A for Color LCD."""
         self._draw_common_data(draw=ImageDraw.Draw(img), scale=2)
 
-    def button_request(self, button: int, request: str = '\n') -> str:
+    def button_request(self, button: LcdButton, request: str = '\n') -> str:
         """
         Prepare AV-8B N/A specific DCS-BIOS request for button pressed.
 
         For G13/G15/G510: 1-4
         For G19 9-15: LEFT = 9, RIGHT = 10, OK = 11, CANCEL = 12, UP = 13, DOWN = 14, MENU = 15
-        :param button: possible values 1-4
+
+        :param button: LcdButton Enum
         :param request: valid DCS-BIOS command as string
         :return: ready to send DCS-BIOS request
         """
-        action = {1: 'UFC_COM1_SEL -3200\n',
-                  2: 'UFC_COM1_SEL 3200\n',
-                  3: 'UFC_COM2_SEL -3200\n',
-                  4: 'UFC_COM2_SEL 3200\n',
-                  9: 'UFC_COM1_SEL -3200\n',
-                  10: 'UFC_COM1_SEL 3200\n',
-                  14: 'UFC_COM2_SEL -3200\n',
-                  13: 'UFC_COM2_SEL 3200\n'}
+        action = {LcdButton.ONE: 'UFC_COM1_SEL -3200\n',
+                  LcdButton.TWO: 'UFC_COM1_SEL 3200\n',
+                  LcdButton.THREE: 'UFC_COM2_SEL -3200\n',
+                  LcdButton.FOUR: 'UFC_COM2_SEL 3200\n',
+                  LcdButton.LEFT: 'UFC_COM1_SEL -3200\n',
+                  LcdButton.RIGHT: 'UFC_COM1_SEL 3200\n',
+                  LcdButton.DOWN: 'UFC_COM2_SEL -3200\n',
+                  LcdButton.UP: 'UFC_COM2_SEL 3200\n'}
         return super().button_request(button, action.get(button, '\n'))
