@@ -12,21 +12,6 @@ LOG = getLogger(__name__)
 TYPE_MONO = 1
 TYPE_COLOR = 2
 
-# LCD Monochrome buttons
-MONO_BUTTON_0 = 0x1
-MONO_BUTTON_1 = 0x2
-MONO_BUTTON_2 = 0x4
-MONO_BUTTON_3 = 0x8
-
-# LCD Color buttons
-COLOR_BUTTON_LEFT = 0x0100
-COLOR_BUTTON_RIGHT = 0x0200
-COLOR_BUTTON_OK = 0x0400
-COLOR_BUTTON_CANCEL = 0x0800
-COLOR_BUTTON_UP = 0x1000
-COLOR_BUTTON_DOWN = 0x2000
-COLOR_BUTTON_MENU = 0x4000
-
 # LCD Monochrome size
 MONO_WIDTH = 160
 MONO_HEIGHT = 43
