@@ -3,9 +3,9 @@ import struct
 from collections import deque
 from importlib import import_module
 from logging import getLogger
+from threading import Event
 from time import time, gmtime
 from typing import Iterator
-from threading import Event
 
 from dcspy import RECV_ADDR, MULTICAST_IP
 from dcspy.dcsbios import ProtocolParser
@@ -14,7 +14,7 @@ from dcspy.utils import check_ver_at_github
 
 LOG = getLogger(__name__)
 LOOP_FLAG = True
-__version__ = '1.7.0'
+__version__ = '1.7.3'
 
 
 def _handle_connection(lcd: LogitechKeyboard, parser: ProtocolParser, sock: socket.socket, event: Event) -> None:
@@ -104,4 +104,8 @@ def dcspy_run(lcd_type: str, event: Event) -> None:
     lcd = getattr(import_module('dcspy.logitech'), lcd_type)(parser)
     LOG.info(f'Loading: {str(lcd)}')
     LOG.debug(f'Loading: {repr(lcd)}')
-    _handle_connection(lcd, parser, _prepare_socket(), event)
+    dcs_sock = _prepare_socket()
+    _handle_connection(lcd, parser, dcs_sock, event)
+    dcs_sock.close()
+    LOG.info('DCSpy stopped.')
+    lcd.display = ['Logitech LCD OK', 'DCSpy stopped', '', f'v{__version__}']
